

#    experiment run control for AIDA - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0


variable MaxASICs 4
variable MaxChans 16

variable Options


proc Dump_Raw_Data {} {return [Dump_Raw_Aida_data]}

proc set_Options {} {
   variable Options

   load_Options

   if {![info exists Options(Aida_GroupBase)]} {set Options(Aida_GroupBase) 1}
   if {![info exists Options(Include.Aida)]} {set Options(Include.Aida) 1}

   if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
   if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
   if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
   if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

   if {![info exists Options(Aida_Hist_L_Enable)]} {set Options(Aida_Hist_L_Enable) 1}
   if {![info exists Options(Aida_Hist_H_Enable)]} {set Options(Aida_Hist_H_Enable) 1}
   if {![info exists Options(Aida_Hist_D_Enable)]} {set Options(Aida_Hist_D_Enable) 1}

   if {![info exists Options(Aida.channels)]} {set Options(Aida.channels) 65536}
   if {![info exists Options(Aida.offset)]} {set Options(Aida.offset) 0}
   if {![info exists Options(Aida.shift)]} {set Options(Aida.shift) 0}

   update_Options

}


proc Dump_Raw_Aida_Data {} {

   set info "ASIC data $::DataAcquisition::ACQSERVER\n"

   for {set offset 0} {$offset <= 300} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "17 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
         set w [expr ($word >> 30) & 3]
         set det [expr ($word >> 27) & 7]
         set chan [expr ($word >> 20) & 31]
         if {$chan < 16} {set range L} else {set range H; set chan [expr $chan & 15]}
         set dat [format 0x%4x [expr $word & 0x0000ffff]]

         append info "Word $w: ASIC $det: channel $chan: range $range: "

         switch $w {
            0 {append info "ADC data $dat"}
            1 {append info "Time 47-32 $dat"}
            2 {append info "Time 31-16 $dat"}
            3 {append info "Time 15-0 $dat"}
         }
         append info "\n"
     }
   }

   return "$info"
}


proc create_Aida_spectra {} {
   variable MaxASICs
   variable MaxChans 
   variable Options

   set Me $::DataAcquisition::ACQSERVER

     puts "Creating spectra for $Me"

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

      puts "Creating spectra for ASIC $Det"

      for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

        if {$Options(Aida_Hist_L_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.L $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum $Det.$Ch.L"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.L [set Me]_$Det.$Ch.L} m]

         }

        if {$Options(Aida_Hist_H_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.H $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum $Det.$Ch.H"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.H [set Me]_$Det.$Ch.H} m]

         }
     }
   }

#  for the discriminator data (hit) and its rate 

        if {$Options(Aida_Hist_D_Enable) == 1} {

        set z [catch {CreateSpectrum1D Hit 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum Hit"
            error $m
         }

        set z [catch {SpectrumPutTitle Hit [set Me]_Hit} m]

        set z [catch {CreateSpectrum1D HitRate 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Attempting to create spectrum HitRate"
            error $m
         }

        set z [catch {SpectrumPutTitle HitRate [set Me]_HitRate} m]

        }
}

proc map_Aida_spectra {} {
   variable MaxASICs 
   variable MaxChans
   variable Options
   upvar #0 SpectraHandle f

   set Me $::DataAcquisition::ACQSERVER

   set SpectraDefs [file join /tmp SpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          puts "Cannot open file $SpectraDefs $f"
          return
      }

      map_spectrum "Stat 0 0 0 $Options(Stat.shift)" 

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

     for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

       if {$Options(Aida_Hist_L_Enable) == 1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.L $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }

       if {$Options(Aida_Hist_H_Enable) ==1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+16+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.H $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }
     }
   }

   if {$Options(Aida_Hist_D_Enable) == 1} {
        map_spectrum "Hit [expr $Options(Aida_GroupBase)+128] 0 0 0"
   }

   close $f
}

proc doSetUp {} {
    variable Options

puts "executing doSetUp"

     set_Options; make-UIregisters

     DataAcqCommand 1 "Halt"

     unmap_spectra

     delete_spectra

      set Me $::DataAcquisition::ACQSERVER

      CreateSpectrum1D Stat $Options(Stat.channels)
      set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]

      CreateSpectrum1D Rate $Options(Rate.channels)
      set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]

    create_Aida_spectra

    map_Aida_spectra 

    map_spectra

    Setup_Electronics

     DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout

    puts "completed doSetUp"

}

proc Reset_Electronics {} {    ;#   used by RESET

     WriteAIDARegister 0x400 0   	 ;#   disable Timestamp counter
     WriteAIDARegister 0x300 0x31
     WriteAIDARegister 0x300 0x1
     WriteAIDARegister 0x300 0x4000   ;#    unset enable readout and reset 

}

proc Setup_Electronics {} {    ;#   used by SETUP

   WriteAIDARegister 0x000 3
#   WriteAIDARegister 0x50e 7			;# disable the SYNC et al for the DISC stream

   WriteAIDARegister 0x007 3        ;#  enable MBS drivers for CLOCK and RESET
   WriteAIDARegister 0x315 0x180    ;#  MBS triggger delay
   WriteAIDARegister 0x307 0xF      ;# enable readout for all four of the ASICs

}

proc Stop_Electronics {} {

     WriteAIDARegister 0x300 0   ;#    unset enable readout
     WriteAIDARegister 0x310 0   ;#    unset enable MBS
#     WriteAIDARegister 0x307 0   ; # unset the enable for readout of all the ASICs
     WriteAIDARegister 0x500 0   ;#    unset enable readout
}

proc Go_Electronics {} {

     WriteAIDARegister 0x400 0   ;#   disable Timestamp counter
     WriteAIDARegister 0x404 0   ;#   set Timestamp low 32 bits 
     WriteAIDARegister 0x405 0   ;#   set Timestamp high 32 bits
     WriteAIDARegister 0x400 1   ;#   enable Timestamp counter

 #    WriteAIDARegister 0x300 0x4000   ;#  set RDO_reset; unset enable readout
 #    WriteAIDARegister 0x500 0x4000   ;#  set RDO_reset; unset enable readout


     WriteAIDARegister 0x300 0x0000   ;#  unset RDO_reset
     WriteAIDARegister 0x500 0x0000   ;#  unset RDO_reset

     after 1000			 ; # Pause 1 second to allow the readout to settle

     WriteAIDARegister 0x300 0x0001   ;#  unset RDO_reset; set enable readout
     WriteAIDARegister 0x310 1        ;#   enable MBS
     WriteAIDARegister 0x500 0x0001   ;#  unset RDO_reset; set enable readout
#     WriteAIDARegister 0x307 0xF      ; # enable readout for all four of the ASICs

}


#    set f [file join $env(MIDASBASE) TclHttpd Html AIDA RunControl Options.tcl]
#    set z [catch {source $f} m]
#    if {$z == 0} {puts "RunControl loaded $f"} else {puts "RunControl returned z=$z and $m"}

    make-UIregisters

    Set_Options




